@@ -216,7 +216,7 @@ class EMConstant(Constant):
 
     @property
     def cgs(self):
-        """Overridden for EMConstant to raise a `~.exceptions.TypeError`
+        """Overridden for EMConstant to raise a `TypeError`
         emphasizing that there are multiple EM extensions to CGS.
         """
 
