@@ -15,7 +15,6 @@ from ..angles import Longitude, Latitude, Angle
 from ...tests.helper import pytest
 from ... import units as u
 from ..errors import IllegalSecondError, IllegalMinuteError, IllegalHourError
-from ...utils.compat import NUMPY_LT_1_7
 
 
 def test_create_angles():
@@ -869,14 +868,6 @@ def test_repr_latex():
     arrangle = Angle([1, 2.1], u.deg)
     rlarrangle = arrangle._repr_latex_()
 
-    if NUMPY_LT_1_7:
-        # numpy 1.6 gives weird latex output for unclear reasons.  We don't care
-        # that much though, so just xfail after making sure it isn't
-        # over-backslashing
-        assert '\\\\' not in rlscangle
-        assert '\\\\' not in rlarrangle
-        pytest.xfail('numpy 1.6 does not give correct to_string latex output')
-
     assert rlscangle == '$2^\circ06{}^\prime00{}^{\prime\prime}$'
     assert rlscangle.split('$')[1] in rlarrangle
 
