@@ -27,7 +27,7 @@ from ... import coordinates as coords
 from ..errors import *
 
 try:
-    import scipy
+    import scipy  # pylint: disable=W0611
 except ImportError:
     HAS_SCIPY = False
 else:
