@@ -4,6 +4,7 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+from copy import deepcopy
 import numpy as np
 
 from ... import units as u
@@ -12,9 +13,20 @@ from ...tests.helper import (pytest, quantity_allclose as allclose,
                              assert_quantity_allclose as assert_allclose)
 from ...utils import OrderedDescriptorContainer
 from .. import representation
+from ..representation import REPRESENTATION_CLASSES
 
 NUMPY_LT_1P7 = [int(x) for x in np.__version__.split('.')[:2]] < [1, 7]
 
+
+def setup_function(func):
+    func.REPRESENTATION_CLASSES_ORIG = deepcopy(REPRESENTATION_CLASSES)
+
+
+def teardown_function(func):
+    REPRESENTATION_CLASSES.clear()
+    REPRESENTATION_CLASSES.update(func.REPRESENTATION_CLASSES_ORIG)
+
+
 def test_frame_attribute_descriptor():
     """ Unit tests of the FrameAttribute descriptor """
     from ..baseframe import FrameAttribute
@@ -188,7 +200,6 @@ def test_converting_units():
     import re
     from ..baseframe import RepresentationMapping
     from ..builtin_frames import ICRS, FK5
-    from ..representation import SphericalRepresentation
 
     # this is a regular expression that with split (see below) removes what's
     # the decimal point  to fix rounding problems
@@ -364,7 +375,6 @@ def test_time_inputs():
     """
     from ...time import Time
     from ..builtin_frames import FK4
-    from ...utils.exceptions import AstropyWarning
 
     c = FK4(1 * u.deg, 2 * u.deg, equinox='J2001.5', obstime='2000-01-01 12:00:00')
     assert c.equinox == Time('J2001.5')
@@ -521,9 +531,11 @@ def test_nodata_error():
     from ..builtin_frames import ICRS
 
     i = ICRS()
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError) as excinfo:
         i.data
 
+    assert 'does not have associated data' in str(excinfo.value)
+
 def test_len0_data():
     from ..builtin_frames import ICRS
 
@@ -663,3 +675,24 @@ def test_getitem_representation():
     c = ICRS([1, 1] * u.deg, [2, 2] * u.deg)
     c.representation = 'cartesian'
     assert c[0].representation is representation.CartesianRepresentation
+
+
+def test_component_error_useful():
+    """
+    Check that a data-less frame gives useful error messages about not having
+    data when the attributes asked for are possible coordinate components
+    """
+    from ..builtin_frames import ICRS
+
+    i = ICRS()
+
+    with pytest.raises(ValueError) as excinfo:
+        i.ra
+    assert 'does not have associated data' in str(excinfo.value)
+
+    with pytest.raises(AttributeError) as excinfo1:
+        i.foobar
+    with pytest.raises(AttributeError) as excinfo2:
+        i.lon  # lon is *not* the component name despite being the underlying representation's name
+    assert "object has no attribute 'foobar'" in str(excinfo1.value)
+    assert "object has no attribute 'lon'" in str(excinfo2.value)
