@@ -21,12 +21,13 @@ from ...tests.helper import (pytest, remote_data, catch_warnings,
 from ..representation import REPRESENTATION_CLASSES
 from ...coordinates import (ICRS, FK4, FK5, Galactic, SkyCoord, Angle,
                             SphericalRepresentation, CartesianRepresentation,
-                            UnitSphericalRepresentation, AltAz)
-from ...coordinates import Latitude, Longitude, EarthLocation
+                            UnitSphericalRepresentation, AltAz,
+                            BaseCoordinateFrame, FrameAttribute,
+                            frame_transform_graph)
+from ...coordinates import Latitude, EarthLocation
 from ...time import Time
 from ...utils import minversion
-from ...utils.exceptions import AstropyDeprecationWarning
-from ...utils.compat import NUMPY_LT_1_7
+from ...utils.exceptions import AstropyDeprecationWarning, AstropyWarning
 
 RA = 1.0 * u.deg
 DEC = 2.0 * u.deg
@@ -69,13 +70,13 @@ rt_sets = []
 rt_frames = [ICRS, FK4, FK5, Galactic]
 for rt_frame0 in rt_frames:
     for rt_frame1 in rt_frames:
-            for equinox0 in (None, 'J1975.0'):
-                for obstime0 in (None, 'J1980.0'):
-                    for equinox1 in (None, 'J1975.0'):
-                        for obstime1 in (None, 'J1980.0'):
-                            rt_sets.append((rt_frame0, rt_frame1,
-                                            equinox0, equinox1,
-                                            obstime0, obstime1))
+        for equinox0 in (None, 'J1975.0'):
+            for obstime0 in (None, 'J1980.0'):
+                for equinox1 in (None, 'J1975.0'):
+                    for obstime1 in (None, 'J1980.0'):
+                        rt_sets.append((rt_frame0, rt_frame1,
+                                        equinox0, equinox1,
+                                        obstime0, obstime1))
 rt_args = ('frame0','frame1','equinox0','equinox1','obstime0','obstime1')
 
 
@@ -482,8 +483,6 @@ def test_seps():
     assert sep3d == 1 * u.kpc
 
 def test_repr():
-    # Repr tests must use exact floating point vals because Python 2.6
-    # outputs values like 0.1 as 0.1000000000001.  No workaround found.
     sc1 = SkyCoord(0 * u.deg, 1 * u.deg, frame='icrs')
     sc2 = SkyCoord(1 * u.deg, 1 * u.deg, frame='icrs', distance=1 * u.kpc)
 
@@ -500,7 +499,6 @@ def test_repr():
     assert repr(sc_default) == ('<SkyCoord (ICRS): (ra, dec) in deg\n'
                                 '    (0.0, 1.0)>')
 
-@pytest.mark.skipif('NUMPY_LT_1_7')
 def test_repr_altaz():
     sc2 = SkyCoord(1 * u.deg, 1 * u.deg, frame='icrs', distance=1 * u.kpc)
     loc = EarthLocation(-2309223 * u.m, -3695529 * u.m, -4641767 * u.m)
@@ -586,6 +584,14 @@ def test_position_angle():
     assert cicrs.position_angle(cfk5) < 91.0 * u.deg
 
 
+def test_position_angle_directly():
+    """Regression check for #3800: position_angle should accept floats."""
+    from ..angle_utilities import position_angle
+    result = position_angle(10., 20., 10., 20.)
+    assert result.unit is u.radian
+    assert result.value == 0.
+
+
 def test_table_to_coord():
     """
     Checks "end-to-end" use of `Table` with `SkyCoord` - the `Quantity`
@@ -894,6 +900,16 @@ def test_deepcopy():
     assert c5.representation == c4.representation
 
 
+def test_no_copy():
+    c1 = SkyCoord(np.arange(10.) * u.hourangle, np.arange(20., 30.) * u.deg)
+    c2 = SkyCoord(c1, copy=False)
+    # Note: c1.ra and c2.ra will *not* share memory, as these are recalculated
+    # to be in "preferred" units.  See discussion in #4883.
+    assert np.may_share_memory(c1.data.lon, c2.data.lon)
+    c3 = SkyCoord(c1, copy=True)
+    assert not np.may_share_memory(c1.data.lon, c3.data.lon)
+
+
 def test_immutable():
     c1 = SkyCoord(1 * u.deg, 2 * u.deg)
     with pytest.raises(AttributeError):
@@ -1166,3 +1182,73 @@ def test_getitem_representation():
     sc = SkyCoord([1, 1] * u.deg, [2, 2] * u.deg)
     sc.representation = 'cartesian'
     assert sc[0].representation is CartesianRepresentation
+
+def test_spherical_offsets():
+    i00 = SkyCoord(0*u.arcmin, 0*u.arcmin, frame='icrs')
+    i01 = SkyCoord(0*u.arcmin, 1*u.arcmin, frame='icrs')
+    i10 = SkyCoord(1*u.arcmin, 0*u.arcmin, frame='icrs')
+    i11 = SkyCoord(1*u.arcmin, 1*u.arcmin, frame='icrs')
+    i22 = SkyCoord(2*u.arcmin, 2*u.arcmin, frame='icrs')
+
+    dra, ddec = i00.spherical_offsets_to(i01)
+    assert_allclose(dra, 0*u.arcmin)
+    assert_allclose(ddec, 1*u.arcmin)
+
+    dra, ddec = i00.spherical_offsets_to(i10)
+    assert_allclose(dra, 1*u.arcmin)
+    assert_allclose(ddec, 0*u.arcmin)
+
+    dra, ddec = i10.spherical_offsets_to(i01)
+    assert_allclose(dra, -1*u.arcmin)
+    assert_allclose(ddec, 1*u.arcmin)
+
+    dra, ddec = i11.spherical_offsets_to(i22)
+    assert_allclose(ddec, 1*u.arcmin)
+    assert 0*u.arcmin < dra < 1*u.arcmin
+
+    fk5 = SkyCoord(0*u.arcmin, 0*u.arcmin, frame='fk5')
+
+    with pytest.raises(ValueError):
+        # different frames should fail
+        i00.spherical_offsets_to(fk5)
+
+    i1deg = ICRS(1*u.deg, 1*u.deg)
+    dra, ddec = i00.spherical_offsets_to(i1deg)
+    assert_allclose(dra, 1*u.deg)
+    assert_allclose(ddec, 1*u.deg)
+
+def test_frame_attr_changes():
+    """
+    This tests the case where a frame is added with a new frame attribute after
+    a SkyCoord has been created.  This is necessary because SkyCoords get the
+    attributes set at creation time, but the set of attributes can change as
+    frames are added or removed from the transform graph.  This makes sure that
+    everything continues to work consistently.
+    """
+    sc_before = SkyCoord(1*u.deg, 2*u.deg, frame='icrs')
+
+    assert 'fakeattr' not in dir(sc_before)
+
+    class FakeFrame(BaseCoordinateFrame):
+        fakeattr = FrameAttribute()
+
+    # doesn't matter what this does as long as it just puts the frame in the
+    # transform graph
+    transset = (ICRS, FakeFrame, lambda c,f:c)
+    frame_transform_graph.add_transform(*transset)
+    try:
+        assert 'fakeattr' in dir(sc_before)
+        assert sc_before.fakeattr is None
+
+        sc_after1 = SkyCoord(1*u.deg, 2*u.deg, frame='icrs')
+        assert 'fakeattr' in dir(sc_after1)
+        assert sc_after1.fakeattr is None
+
+        sc_after2 = SkyCoord(1*u.deg, 2*u.deg, frame='icrs', fakeattr=1)
+        assert sc_after2.fakeattr == 1
+    finally:
+        frame_transform_graph.remove_transform(*transset)
+
+    assert 'fakeattr' not in dir(sc_before)
+    assert 'fakeattr' not in dir(sc_after1)
+    assert 'fakeattr' not in dir(sc_after2)
