@@ -1,10 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import numpy as np
 
 from ...tests.helper import pytest, pickle_protocol, check_pickling_recovery
-from ...extern.six.moves import cPickle
 from ... import cosmology as cosm
 
 originals = [cosm.FLRW]
