@@ -8,7 +8,8 @@ import copy
 from ....extern.six.moves import cStringIO as StringIO
 from ... import ascii
 from .... import table
-from ....tests.helper import pytest
+from ....tests.helper import pytest, catch_warnings
+from ....utils.exceptions import AstropyWarning
 from .... import units
 
 from .common import setup_function, teardown_function
@@ -98,7 +99,7 @@ ID & XCENTER & YCENTER & MAG & MERR & MSKY & NITER & SHARPNESS & CHI & PIER & PE
 \\tablehead{\\colhead{ID} & \\colhead{XCENTER} & \\colhead{YCENTER} & \\colhead{MAG} & \\colhead{MERR} & \\colhead{MSKY} & \\colhead{NITER} & \\colhead{SHARPNESS} & \\colhead{CHI} & \\colhead{PIER} & \\colhead{PERROR}\\\\ \\colhead{ } & \\colhead{pixels} & \\colhead{pixels} & \\colhead{magnitudes} & \\colhead{magnitudes} & \\colhead{counts} & \\colhead{ } & \\colhead{ } & \\colhead{ } & \\colhead{ } & \\colhead{perrors}}
 \\startdata
 14 & 138.538 & 256.405 & 15.461 & 0.003 & 34.85955 & 4 & -0.032 & 0.802 & 0 & No_error \\\\
-18 & 18.114 & 280.170 & 22.329 & 0.206 & 30.12784 & 4 & -2.544 & 1.104 & 0 & No_error \\\\
+18 & 18.114 & 280.170 & 22.329 & 0.206 & 30.12784 & 4 & -2.544 & 1.104 & 0 & No_error
 \\enddata
 \\end{deluxetable}
 """
@@ -113,7 +114,7 @@ ID & XCENTER & YCENTER & MAG & MERR & MSKY & NITER & SHARPNESS & CHI & PIER & PE
 \\tablehead{\\colhead{ID} & \\colhead{XCENTER} & \\colhead{YCENTER} & \\colhead{MAG} & \\colhead{MERR} & \\colhead{MSKY} & \\colhead{NITER} & \\colhead{SHARPNESS} & \\colhead{CHI} & \\colhead{PIER} & \\colhead{PERROR}\\\\ \\colhead{ } & \\colhead{[pixel]} & \\colhead{pixels} & \\colhead{[mag]} & \\colhead{magnitudes} & \\colhead{counts} & \\colhead{ } & \\colhead{ } & \\colhead{ } & \\colhead{ } & \\colhead{perrors}}
 \\startdata
 14 & 138.538 & 256.405 & 15.461 & 0.003 & 34.85955 & 4 & -0.032 & 0.802 & 0 & No_error \\\\
-18 & 18.114 & 280.170 & 22.329 & 0.206 & 30.12784 & 4 & -2.544 & 1.104 & 0 & No_error \\\\
+18 & 18.114 & 280.170 & 22.329 & 0.206 & 30.12784 & 4 & -2.544 & 1.104 & 0 & No_error
 \\enddata
 \\end{deluxetable*}
 """
@@ -468,6 +469,17 @@ def test_write_comments(fast_writer):
     expected = ['a b c', '1 2 3']
     assert out.getvalue().splitlines() == expected
 
+@pytest.mark.parametrize("fast_writer", [True, False])
+@pytest.mark.parametrize("fmt", ['%0.1f', '.1f', '0.1f', '{0:0.1f}'])
+def test_write_format(fast_writer, fmt):
+    """Check different formats for a column."""
+    data = ascii.read('#c1\n  # c2\t\na,b,c\n#  c3\n1.11,2.22,3.33')
+    out = StringIO()
+    expected = ['# c1', '# c2', '# c3', 'a b c', '1.1 2.22 3.33']
+    data['a'].format = fmt
+    ascii.write(data, out, format='basic', fast_writer=fast_writer)
+    assert out.getvalue().splitlines() == expected
+
 @pytest.mark.parametrize("fast_writer", [True, False])
 def test_strip_names(fast_writer):
     """Names should be stripped of whitespace by default."""
@@ -493,7 +505,7 @@ def test_latex_units():
 \\tablehead{\\colhead{date} & \\colhead{NUV exp.time}\\\\ \\colhead{ } & \\colhead{s}}
 \\startdata
 a & 1 \\\\
-b & 2 \\\\
+b & 2
 \\enddata
 \\end{table}
 '''.replace('\n', os.linesep)
@@ -534,3 +546,53 @@ def test_byte_string_output(fast_writer):
     out = StringIO()
     ascii.write(t, out, fast_writer=fast_writer)
     assert out.getvalue().splitlines() == ['col0', 'Hello', 'World']
+
+
+@pytest.mark.parametrize('names, include_names, exclude_names, formats, issues_warning', [
+    (['x', 'y'], ['x', 'y'], ['x'], {'x':'%d', 'y':'%f'}, True),
+    (['x', 'y'], ['x', 'y'], ['y'], {'x':'%d'}, False),
+    (['x', 'y'], ['x', 'y'], [], {'p':'%d', 'q':'%f'}, True),
+    (['x', 'y'], ['x', 'y'], [], {'z':'%f'}, True),
+    (['x', 'y'], ['x', 'y'], [], {'x':'%d'}, False),
+    (['x', 'y'], ['x', 'y'], [], {'p':'%d', 'y':'%f'}, True),
+    (['x', 'y'], ['x', 'y'], [], {}, False)
+])
+def test_names_with_formats(names, include_names, exclude_names, formats, issues_warning):
+    """Test for #4508."""
+    t = table.Table([[1,2,3],[4.1,5.2,6.3]])
+    with catch_warnings(AstropyWarning) as ASwarn:
+        out = StringIO()
+        ascii.write(t, out, names=names, include_names=include_names,
+        exclude_names=exclude_names, formats=formats)
+    assert (issues_warning == (len(ASwarn) == 1))
+
+
+@pytest.mark.parametrize('formats, issues_warning', [
+    ({'p':'%d', 'y':'%f'}, True),
+    ({'x':'%d', 'y':'%f'}, True),
+    ({'z':'%f'}, True),
+    ({}, False)
+])
+def test_columns_names_with_formats(formats, issues_warning):
+    """Test the fix for #4508."""
+    t = table.Table([[1,2,3],[4.1,5.2,6.3]])
+    with catch_warnings(AstropyWarning) as ASwarn:
+        out = StringIO()
+        ascii.write(t, out,formats=formats)
+    assert (issues_warning == (len(ASwarn) == 1))
+
+
+@pytest.mark.parametrize("fast_writer", [True, False])
+def test_write_quoted_empty_field(fast_writer):
+    """
+    Test the fix for #4350 where byte strings were output with a
+    leading `b` on Py3.
+    """
+    t = table.Table([['Hello', ''], ['', '']], dtype=['S10', 'S10'])
+    out = StringIO()
+    ascii.write(t, out, fast_writer=fast_writer)
+    assert out.getvalue().splitlines() == ['col0 col1', 'Hello ""', '"" ""']
+
+    out = StringIO()
+    ascii.write(t, out, fast_writer=fast_writer, delimiter=',')
+    assert out.getvalue().splitlines() == ['col0,col1', 'Hello,', ',']
