@@ -8,6 +8,7 @@ import warnings
 import weakref
 
 from functools import reduce
+from collections import OrderedDict
 
 import numpy as np
 from numpy import char as chararray
@@ -19,8 +20,9 @@ from .util import (pairwise, _is_int, _convert_array, encode_ascii, cmp,
 from .verify import VerifyError, VerifyWarning
 
 from ...extern.six import string_types, iteritems
-from ...utils import lazyproperty, isiterable, indent, OrderedDict
+from ...utils import lazyproperty, isiterable, indent
 from ...utils.compat import ignored
+from ...utils.exceptions import AstropyDeprecationWarning
 
 
 __all__ = ['Column', 'ColDefs', 'Delayed']
@@ -49,11 +51,17 @@ NUMPY2FITS['b1'] = 'L'
 NUMPY2FITS['u2'] = 'I'
 NUMPY2FITS['u4'] = 'J'
 NUMPY2FITS['u8'] = 'K'
+# Add half precision floating point numbers which will be up-converted to
+# single precision.
+NUMPY2FITS['f2'] = 'E'
 
 # This is the order in which values are converted to FITS types
 # Note that only double precision floating point/complex are supported
 FORMATORDER = ['L', 'B', 'I', 'J', 'K', 'D', 'M', 'A']
 
+# Convert single precision floating point/complex to double precision.
+FITSUPCONVERTERS = {'E' : 'D', 'C' : 'M'}
+
 # mapping from ASCII table TFORM data type to numpy data type
 # A: Character
 # I: Integer (32-bit)
@@ -261,7 +269,7 @@ class _AsciiColumnFormat(_BaseColumnFormat):
     Conversions between the two column formats can be performed using the
     ``to/from_binary`` methods on this class, or the ``to/from_ascii``
     methods on the `_ColumnFormat` class.  But again, not all conversions are
-    possible and may result in a `~.exceptions.ValueError`.
+    possible and may result in a `ValueError`.
     """
 
     def __new__(cls, format, strict=False):
@@ -321,6 +329,9 @@ class _FormatX(str):
         obj.repeat = repeat
         return obj
 
+    def __getnewargs__(self):
+        return (self.repeat,)
+
     @property
     def tform(self):
         return '%sX' % self.repeat
@@ -348,6 +359,9 @@ class _FormatP(str):
         obj.max = max
         return obj
 
+    def __getnewargs__(self):
+        return (self.dtype, self.repeat, self.max)
+
     @classmethod
     def from_tform(cls, format):
         m = cls._format_re.match(format)
@@ -584,6 +598,7 @@ class Column(NotifierMixin):
         else:
             self._physical_values = False
 
+        self._parent_fits_rec = None
         self.array = array
 
     def __repr__(self):
@@ -614,6 +629,123 @@ class Column(NotifierMixin):
 
         return hash((self.name.lower(), self.format))
 
+    @property
+    def array(self):
+        """
+        The Numpy `~numpy.ndarray` associated with this `Column`.
+
+        If the column was instantiated with an array passed to the ``array``
+        argument, this will return that array.  However, if the column is
+        later added to a table, such as via `BinTableHDU.from_columns` as
+        is typically the case, this attribute will be updated to reference
+        the associated field in the table, which may no longer be the same
+        array.
+        """
+
+        # Ideally the .array attribute never would have existed in the first
+        # place, or would have been internal-only.  This is a legacy of the
+        # older design from PyFITS that needs to have continued support, for
+        # now.
+
+        # One of the main problems with this design was that it created a
+        # reference cycle.  When the .array attribute was updated after
+        # creating a FITS_rec from the column (as explained in the docstring) a
+        # reference cycle was created.  This is because the code in BinTableHDU
+        # (and a few other places) does essentially the following:
+        #
+        # data._coldefs = columns  # The ColDefs object holding this Column
+        # for col in columns:
+        #     col.array = data.field(col.name)
+        #
+        # This way each columns .array attribute now points to the field in the
+        # table data.  It's actually a pretty confusing interface (since it
+        # replaces the array originally pointed to by .array), but it's the way
+        # things have been for a long, long time.
+        #
+        # However, this results, in *many* cases, in a reference cycle.
+        # Because the array returned by data.field(col.name), while sometimes
+        # an array that owns its own data, is usually like a slice of the
+        # original data.  It has the original FITS_rec as the array .base.
+        # This results in the following reference cycle (for the n-th column):
+        #
+        #    data -> data._coldefs -> data._coldefs[n] ->
+        #     data._coldefs[n].array -> data._coldefs[n].array.base -> data
+        #
+        # Because ndarray objects do not handled by Python's garbage collector
+        # the reference cycle cannot be broken.  Therefore the FITS_rec's
+        # refcount never goes to zero, its __del__ is never called, and its
+        # memory is never freed.  This didn't occur in *all* cases, but it did
+        # occur in many cases.
+        #
+        # To get around this, Column.array is no longer a simple attribute
+        # like it was previously.  Now each Column has a ._parent_fits_rec
+        # attribute which is a weakref to a FITS_rec object.  Code that
+        # previously assigned each col.array to field in a FITS_rec (as in
+        # the example a few paragraphs above) is still used, however now
+        # array.setter checks if a reference cycle will be created.  And if
+        # so, instead of saving directly to the Column's __dict__, it creates
+        # the ._prent_fits_rec weakref, and all lookups of the column's .array
+        # go through that instead.
+        #
+        # This alone does not fully solve the problem.  Because
+        # _parent_fits_rec is a weakref, if the user ever holds a reference to
+        # the Column, but deletes all references to the underlying FITS_rec,
+        # the .array attribute would suddenly start returning None instead of
+        # the array data.  This problem is resolved on FITS_rec's end.  See the
+        # note in the FITS_rec._coldefs property for the rest of the story.
+
+        # If the Columns's array is not a reference to an existing FITS_rec,
+        # then it is just stored in self.__dict__; otherwise check the
+        # _parent_fits_rec reference if it 's still available.
+        if 'array' in self.__dict__:
+            return self.__dict__['array']
+        elif self._parent_fits_rec is not None:
+            parent = self._parent_fits_rec()
+            if parent is not None:
+                return parent[self.name]
+        else:
+            return None
+
+    @array.setter
+    def array(self, array):
+        # The following looks over the bases of the given array to check if it
+        # has a ._coldefs attribute (i.e. is a FITS_rec) and that that _coldefs
+        # contains this Column itself, and would create a reference cycle if we
+        # stored the array directly in self.__dict__.
+        # In this case it instead sets up the _parent_fits_rec weakref to the
+        # underlying FITS_rec, so that array.getter can return arrays through
+        # self._parent_fits_rec().field(self.name), rather than storing a
+        # hard reference to the field like it used to.
+        base = array
+        while True:
+            if (hasattr(base, '_coldefs') and
+                    isinstance(base._coldefs, ColDefs)):
+                for col in base._coldefs:
+                    if col is self and self._parent_fits_rec is None:
+                        self._parent_fits_rec = weakref.ref(base)
+
+                        # Just in case the user already set .array to their own
+                        # array.
+                        if 'array' in self.__dict__:
+                            del self.__dict__['array']
+                        return
+
+            if getattr(base, 'base', None) is not None:
+                base = base.base
+            else:
+                break
+
+        self.__dict__['array'] = array
+
+    @array.deleter
+    def array(self):
+        try:
+            del self.__dict__['array']
+        except KeyError:
+            pass
+
+        self._parent_fits_rec = None
+
     @ColumnAttribute('TTYPE')
     def name(col, name):
         if name is None:
@@ -1950,10 +2082,6 @@ def _scalar_to_format(value):
     FORMATORDER.
     """
 
-    # TODO: Numpy 1.6 and up has a min_scalar_type() function that can handle
-    # this; in the meantime we have to use our own implementation (which for
-    # now is pretty naive)
-
     # First, if value is a string, try to convert to the appropriate scalar
     # value
     for type_ in (int, float, complex):
@@ -1963,22 +2091,14 @@ def _scalar_to_format(value):
         except ValueError:
             continue
 
-    if isinstance(value, int) and value in (0, 1):
-        # Could be a boolean
-        return 'L'
-    elif isinstance(value, int):
-        for char in ('B', 'I', 'J', 'K'):
-            type_ = np.dtype(FITS2NUMPY[char]).type
-            if type_(value) == value:
-                return char
-    elif isinstance(value, float):
-        # For now just assume double precision
-        return 'D'
-    elif isinstance(value, complex):
-        return 'M'
-    else:
-        return 'A' + str(len(value))
+    numpy_dtype_str = np.min_scalar_type(value).str
+    numpy_dtype_str = numpy_dtype_str[1:]  # Strip endianness
 
+    try:
+        fits_format = NUMPY2FITS[numpy_dtype_str]
+        return FITSUPCONVERTERS.get(fits_format, fits_format)
+    except KeyError:
+        return "A" + str(len(value))
 
 def _cmp_recformats(f1, f2):
     """
