@@ -6,7 +6,6 @@ import bz2
 import io
 import mmap
 import os
-import shutil
 import warnings
 import zipfile
 
@@ -21,11 +20,19 @@ except ImportError:
 from ....io import fits
 from ....tests.helper import pytest, raises, catch_warnings, ignore_warnings
 from ....utils.exceptions import AstropyDeprecationWarning
+from ....utils.data import get_pkg_data_filename
 from . import FitsTestCase
 from ..convenience import _getext
 from ..diff import FITSDiff
 from ..file import _File, GZIP_MAGIC
 
+try:
+    import pathlib
+except ImportError:
+    HAS_PATHLIB = False
+else:
+    HAS_PATHLIB = True
+
 
 class TestCore(FitsTestCase):
     def test_with_statement(self):
@@ -66,6 +73,22 @@ class TestCore(FitsTestCase):
         with fits.open(self.temp('test.fits')) as p:
             assert p[1].data[1]['foo'] == 60000.0
 
+    @pytest.mark.skipif('not HAS_PATHLIB')
+    def test_fits_file_path_object(self):
+        """
+        Testing when fits file is passed as pathlib.Path object #4412.
+        """
+        fpath = pathlib.Path(get_pkg_data_filename('data/tdim.fits'))
+        hdulist = fits.open(fpath)
+
+        assert hdulist[0].filebytes() == 2880
+        assert hdulist[1].filebytes() == 5760
+
+        hdulist2 = fits.open(self.data('tdim.fits'))
+
+        assert FITSDiff(hdulist2, hdulist).identical is True
+
+
     def test_add_del_columns(self):
         p = fits.ColDefs([])
         p.add_col(fits.Column(name='FOO', format='3J'))
@@ -682,9 +705,6 @@ class TestFileFunctions(FitsTestCase):
         This tests reading from a ``GzipFile`` object from Astropy's
         compatibility copy of the ``gzip`` module.
         """
-
-        from ....utils.compat import gzip
-
         gf = gzip.GzipFile(self._make_gzip_file())
         try:
             assert len(fits.open(gf)) == 5
