@@ -2,8 +2,7 @@
 
 from __future__ import division
 
-import errno
-import gzip as _system_gzip
+import gzip
 import itertools
 import io
 import mmap
@@ -36,13 +35,8 @@ from ...extern.six import (string_types, integer_types, text_type,
 from ...extern.six.moves import zip
 from ...utils import wraps
 from ...utils.compat import ignored
-from ...utils.compat import gzip as _astropy_gzip
 from ...utils.exceptions import AstropyUserWarning
 
-
-_GZIP_FILE_TYPES = (_astropy_gzip.GzipFile, _system_gzip.GzipFile)
-
-
 if six.PY3:
     cmp = lambda a, b: (a > b) - (a < b)
 elif six.PY2:
@@ -267,12 +261,6 @@ def ignore_sigint(func):
     return wrapped
 
 
-def first(iterable):
-    """Returns the first element from an iterable."""
-
-    return next(iter(iterable))
-
-
 def pairwise(iterable):
     """Return the items of an iterable paired with its next item.
 
@@ -424,7 +412,7 @@ def fileobj_name(f):
 
     if isinstance(f, string_types):
         return f
-    elif isinstance(f, _GZIP_FILE_TYPES):
+    elif isinstance(f, gzip.GzipFile):
         # The .name attribute on GzipFiles does not always represent the name
         # of the file being read/written--it can also represent the original
         # name of the file being compressed
@@ -445,10 +433,16 @@ def fileobj_name(f):
 
 def fileobj_closed(f):
     """
-    Returns True if the given file-like object is closed or if f is not a
-    file-like object.
+    Returns True if the given file-like object is closed or if f is a string
+    (and assumed to be a pathname).
+
+    Returns False for all other types of objects, under the assumption that
+    they are file-like objects with no sense of a 'closed' state.
     """
 
+    if isinstance(f, string_types):
+        return True
+
     if hasattr(f, 'closed'):
         return f.closed
     elif hasattr(f, 'fileobj') and hasattr(f.fileobj, 'closed'):
@@ -492,11 +486,11 @@ def _fileobj_normalize_mode(f):
     # normalize it for them:
     mode = f.mode
 
-    if isinstance(f, _GZIP_FILE_TYPES):
+    if isinstance(f, gzip.GzipFile):
         # GzipFiles can be either readonly or writeonly
-        if mode == _system_gzip.READ:
+        if mode == gzip.READ:
             return 'rb'
-        elif mode == _system_gzip.WRITE:
+        elif mode == gzip.WRITE:
             return 'wb'
         else:
             # This shouldn't happen?
@@ -586,10 +580,18 @@ if sys.platform.startswith('win32'):
         if msvcrt_dll is None:
             # If for some reason the C runtime can't be located then we're dead
             # in the water.  Just return a dummy function
-            return _dummy_is_append_mode
-
-        msvcrt = cdll.LoadLibrary(msvcrt_dll)
-
+            try:
+                # Python3.5 uses msvc14 and it is "recommended" to use
+                # cdll.msvcrt directly. Not sure if that may cause problems.
+                # https://bugs.python.org/issue26727
+                msvcrt = cdll.msvcrt
+            except:
+                # TODO: This except probably should need some Exceptions to
+                # catch ... but since we return a Warning the user is likely
+                # to know that there is something wrong altogether.
+                return _dummy_is_append_mode
+        else:
+            msvcrt = cdll.LoadLibrary(msvcrt_dll)
 
         # Constants
         IOINFO_L2E = 5
@@ -598,7 +600,6 @@ if sys.platform.startswith('win32'):
         FAPPEND = 0x20
         _NO_CONSOLE_FILENO = -2
 
-
         # Types
         intptr_t = POINTER(c_int)
 
@@ -631,7 +632,7 @@ if sys.platform.startswith('win32'):
         def _is_append_mode(fd):
             global _sizeof_ioinfo
             if fd != _NO_CONSOLE_FILENO:
-                idx1 = fd >> IOINFO_L2E # The index into the __pioinfo array
+                idx1 = fd >> IOINFO_L2E  # The index into the __pioinfo array
                 # The n-th ioinfo pointer in __pioinfo[idx1]
                 idx2 = fd & ((1 << IOINFO_L2E) - 1)
                 if 0 <= idx1 < IOINFO_ARRAYS and __pioinfo[idx1] is not None:
