@@ -1,7 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+# TEST_UNICODE_LITERALS
+
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import io
+
 # THIRD-PARTY
 import numpy as np
 from numpy.testing import assert_array_equal
@@ -10,7 +14,10 @@ from numpy.testing import assert_array_equal
 from .. import converters
 from .. import exceptions
 from .. import tree
-from ....tests.helper import raises
+
+from ..table import parse_single_table
+from ....tests.helper import raises, catch_warnings
+from ....utils.data import get_pkg_data_filename
 
 
 @raises(exceptions.E13)
@@ -20,16 +27,18 @@ def test_invalid_arraysize():
     converters.get_converter(field)
 
 
-def test_oversize_char(recwarn):
+def test_oversize_char():
     config = {'pedantic': True}
-    field = tree.Field(
-        None, name='c', datatype='char',
-        config=config)
-    c = converters.get_converter(field, config=config)
-    w = recwarn.pop(exceptions.W47)
+    with catch_warnings(exceptions.W47) as w:
+        field = tree.Field(
+            None, name='c', datatype='char',
+            config=config)
+        c = converters.get_converter(field, config=config)
+    assert len(w) == 1
 
-    c.parse("XXX")
-    w = recwarn.pop(exceptions.W46)
+    with catch_warnings(exceptions.W46) as w:
+        c.parse("XXX")
+    assert len(w) == 1
 
 
 def test_char_mask():
@@ -41,15 +50,16 @@ def test_char_mask():
     assert c.output("Foo", True) == ''
 
 
-def test_oversize_unicode(recwarn):
+def test_oversize_unicode():
     config = {'pedantic': True}
-    field = tree.Field(
-        None, name='c2', datatype='unicodeChar',
-        config=config)
-    c = converters.get_converter(field, config=config)
+    with catch_warnings(exceptions.W46) as w:
+        field = tree.Field(
+            None, name='c2', datatype='unicodeChar',
+            config=config)
+        c = converters.get_converter(field, config=config)
 
-    c.parse("XXX")
-    w = recwarn.pop(exceptions.W46)
+        c.parse("XXX")
+    assert len(w) == 1
 
 
 def test_unicode_mask():
@@ -155,12 +165,13 @@ def test_bit():
 
 def test_bit_mask(recwarn):
     config = {'pedantic': True}
-    field = tree.Field(
-        None, name='c', datatype='bit',
-        config=config)
-    c = converters.get_converter(field, config=config)
-    c.output(True, True)
-    recwarn.pop(exceptions.W39)
+    with catch_warnings(exceptions.W39) as w:
+        field = tree.Field(
+            None, name='c', datatype='bit',
+            config=config)
+        c = converters.get_converter(field, config=config)
+        c.output(True, True)
+    assert len(w) == 1
 
 
 @raises(exceptions.E05)
@@ -226,4 +237,38 @@ def test_float_default_precision():
         config=config)
     c = converters.get_converter(field, config=config)
     assert (c.output([1, 2, 3, 8.999999], [False, False, False, False]) ==
-            '1 2 3 8.999999')
+            '1 2 3 8.9999990000000007')
+
+
+def test_vararray():
+    votable = tree.VOTableFile()
+    resource = tree.Resource()
+    votable.resources.append(resource)
+    table = tree.Table(votable)
+    resource.tables.append(table)
+
+    tabarr = []
+    heads = ['headA', 'headB', 'headC']
+    types = ["char", "double", "int"]
+
+    vals = [["A", 1.0, 2],
+            ["B", 2.0, 3],
+            ["C", 3.0, 4]]
+    for i in range(len(heads)):
+        tabarr.append(tree.Field(
+            votable, name=heads[i], datatype=types[i], arraysize="*"))
+
+    table.fields.extend(tabarr)
+    table.create_arrays(len(vals))
+    for i in range(len(vals)):
+        values = tuple(vals[i])
+        table.array[i] = values
+    buff = io.BytesIO()
+    votable.to_xml(buff)
+
+def test_gemini_v1_2():
+    '''
+    see Pull Request 4782 or Issue 4781 for details
+    '''
+    table = parse_single_table(get_pkg_data_filename('data/gemini.xml'))
+    assert table is not None
