@@ -14,7 +14,6 @@ from . import config as _config
 from . import conf as _conf
 from .extern.six import PY3, text_type
 from .utils import find_current_module
-from .utils.console import color_print
 from .utils.exceptions import AstropyWarning, AstropyUserWarning
 
 __all__ = ['Conf', 'conf', 'log', 'AstropyLogger', 'LoggingError']
@@ -85,18 +84,6 @@ class Conf(_config.ConfigNamespace):
 conf = Conf()
 
 
-LOG_LEVEL = _config.ConfigAlias('0.4', 'LOG_LEVEL', 'log_level')
-USE_COLOR = _config.ConfigAlias(
-    '0.4', 'USE_COLOR', 'use_color', 'astropy.logger', 'astropy')
-LOG_WARNINGS = _config.ConfigAlias('0.4', 'LOG_WARNINGS', 'log_warnings')
-LOG_EXCEPTIONS = _config.ConfigAlias('0.4', 'LOG_EXCEPTIONS', 'log_exceptions')
-LOG_TO_FILE = _config.ConfigAlias('0.4', 'LOG_TO_FILE', 'log_to_file')
-LOG_FILE_PATH = _config.ConfigAlias('0.4', 'LOG_FILE_PATH', 'log_file_path')
-LOG_FILE_LEVEL = _config.ConfigAlias('0.4', 'LOG_FILE_LEVEL', 'log_file_level')
-LOG_FILE_FORMAT = _config.ConfigAlias(
-    '0.4', 'LOG_FILE_FORMAT', 'log_file_format')
-
-
 def _init_log():
     """Initializes the Astropy log--in most circumstances this is called
     automatically when importing astropy.
@@ -430,7 +417,7 @@ class AstropyLogger(Logger):
                 # your code here
         '''
 
-        fh = FileHandler(filename)
+        fh = logging.FileHandler(filename)
         if filter_level is not None:
             fh.setLevel(filter_level)
         if filter_origin is not None:
@@ -485,12 +472,6 @@ class AstropyLogger(Logger):
         yield lh.log_list
         self.removeHandler(lh)
 
-    def setLevel(self, level):
-        """
-        Set the logging level of this logger.
-        """
-        self.level = _checkLevel(level)
-
     def _set_defaults(self):
         '''
         Reset logger to its initial state
@@ -532,7 +513,7 @@ class AstropyLogger(Logger):
                 else:
                     log_file_path = os.path.expanduser(log_file_path)
 
-                fh = FileHandler(log_file_path)
+                fh = logging.FileHandler(log_file_path)
             except (IOError, OSError) as e:
                 warnings.warn(
                     'log file {0!r} could not be opened for writing: '
@@ -550,31 +531,6 @@ class AstropyLogger(Logger):
             self.enable_exception_logging()
 
 
-# The following function is copied from the source code of Python 2.7 and 3.2.
-# This function is not included in Python 2.6 and 3.1, so we have to include it
-# here to provide uniform behavior across versions.
-def _checkLevel(level):
-    '''
-    '''
-    if isinstance(level, int):
-        rv = level
-    elif str(level) == level:
-        if sys.version_info[:2] >= (3, 4):
-            names = logging._nameToLevel
-        else:
-            names = logging._levelNames
-        if level not in names:
-            raise ValueError("Unknown level: %r" % level)
-        rv = names[level]
-    else:
-        raise TypeError("Level not an integer or a valid string: %r" % level)
-    return rv
-
-
-# We now have to be sure that we overload the setLevel in FileHandler, again
-# for compatibility with Python 2.6 and 3.1.
-
-
 class StreamHandler(logging.StreamHandler):
     """
     A specialized StreamHandler that logs INFO and DEBUG messages to
@@ -593,26 +549,22 @@ class StreamHandler(logging.StreamHandler):
 
         if record.levelno < logging.DEBUG or not _conf.use_color:
             print(record.levelname, end='', file=stream)
-        elif(record.levelno < logging.INFO):
-            color_print(record.levelname, 'magenta', end='', file=stream)
-        elif(record.levelno < logging.WARN):
-            color_print(record.levelname, 'green', end='', file=stream)
-        elif(record.levelno < logging.ERROR):
-            color_print(record.levelname, 'brown', end='', file=stream)
         else:
-            color_print(record.levelname, 'red', end='', file=stream)
+            # Import utils.console only if necessary and at the latest because
+            # the import takes a significant time [#4649]
+            from .utils.console import color_print
+            if record.levelno < logging.INFO:
+                color_print(record.levelname, 'magenta', end='', file=stream)
+            elif record.levelno < logging.WARN:
+                color_print(record.levelname, 'green', end='', file=stream)
+            elif record.levelno < logging.ERROR:
+                color_print(record.levelname, 'brown', end='', file=stream)
+            else:
+                color_print(record.levelname, 'red', end='', file=stream)
         record.message = "{0} [{1:s}]".format(record.msg, record.origin)
         print(": " + record.message, file=stream)
 
 
-class FileHandler(logging.FileHandler):
-    def setLevel(self, level):
-        """
-        Set the logging level of this handler.
-        """
-        self.level = _checkLevel(level)
-
-
 class FilterOrigin(object):
     '''A filter for the record origin'''
     def __init__(self, origin):
@@ -631,9 +583,3 @@ class ListHandler(logging.Handler):
 
     def emit(self, record):
         self.log_list.append(record)
-
-    def setLevel(self, level):
-        """
-        Set the logging level of this handler.
-        """
-        self.level = _checkLevel(level)
