@@ -40,9 +40,12 @@ References
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+import warnings
+
 import numpy as np
 
 from ..utils.compat.funcsigs import signature
+from ..utils.exceptions import AstropyUserWarning
 
 # TODO: implement other fitness functions from appendix B of Scargle 2012
 
@@ -85,7 +88,7 @@ def bayesian_blocks(t, x=None, sigma=None,
           :math:`-\ln({\tt gamma})`.
 
         In all three cases, if more than one of ``p0``, ``gamma``, and
-        ``ncp_prior`` is chosen, ``ncp_prior`` takes precendence over ``gamma``
+        ``ncp_prior`` is chosen, ``ncp_prior`` takes precedence over ``gamma``
         which takes precedence over ``p0``.
 
         Alternatively, the fitness parameter can be an instance of
@@ -194,7 +197,6 @@ class FitnessFunc(object):
         self.gamma = gamma
         self.ncp_prior = ncp_prior
 
-
     def validate_input(self, t, x=None, sigma=None):
         """Validate inputs to the model.
 
@@ -337,9 +339,9 @@ class FitnessFunc(object):
         # Compute ncp_prior if not defined
         if self.ncp_prior is None:
             ncp_prior = self.compute_ncp_prior(N)
-        #-----------------------------------------------------------------
+        # ----------------------------------------------------------------
         # Start with first data cell; add one cell at each iteration
-        #-----------------------------------------------------------------
+        # ----------------------------------------------------------------
         for R in range(N):
             # Compute fit_vec : fitness of putative last block (end at R)
             kwds = {}
@@ -374,9 +376,9 @@ class FitnessFunc(object):
             last[R] = i_max
             best[R] = A_R[i_max]
 
-        #-----------------------------------------------------------------
+        # ----------------------------------------------------------------
         # Now find changepoints by iteratively peeling off the last block
-        #-----------------------------------------------------------------
+        # ----------------------------------------------------------------
         change_points = np.zeros(N, dtype=int)
         i_cp = N
         ind = N
@@ -417,12 +419,11 @@ class Events(FitnessFunc):
     """
     def __init__(self, p0=0.05, gamma=None, ncp_prior=None):
         if p0 is not None and gamma is None and ncp_prior is None:
-            import warnings
             warnings.warn('p0 does not seem to accurately represent the false '
                           'positive rate for event data. It is highly '
                           'recommended that you run random trials on signal-'
                           'free noise to calibrate ncp_prior to achieve a '
-                          'desired false positive rate.')
+                          'desired false positive rate.', AstropyUserWarning)
         super(Events, self).__init__(p0, gamma, ncp_prior)
 
     def fitness(self, N_k, T_k):
@@ -474,9 +475,8 @@ class RegularEvents(FitnessFunc):
 
         eps = 1E-8
         if np.any(N_over_M > 1 + eps):
-            import warnings
             warnings.warn('regular events: N/M > 1.  '
-                          'Is the time step correct?')
+                          'Is the time step correct?', AstropyUserWarning)
 
         one_m_NM = 1 - N_over_M
         N_over_M[N_over_M <= 0] = 1
@@ -511,4 +511,3 @@ class PointMeasures(FitnessFunc):
         if x is None:
             raise ValueError("x must be specified for point measures")
         return super(PointMeasures, self).validate_input(t, x, sigma)
-
