@@ -22,8 +22,8 @@ from ...utils.misc import NumpyRNGContext
 
 
 def test_sigma_clip():
-    #need to seed the numpy RNG to make sure we don't get some amazingly flukey
-    #random number that breaks one of the tests
+    # need to seed the numpy RNG to make sure we don't get some
+    # amazingly flukey random number that breaks one of the tests
 
     with NumpyRNGContext(12345):
         # Amazing, I've got the same combination on my luggage!
@@ -68,8 +68,8 @@ def test_sigma_clip():
 
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_compare_to_scipy_sigmaclip():
-    #need to seed the numpy RNG to make sure we don't get some amazingly flukey
-    #random number that breaks one of the tests
+    # need to seed the numpy RNG to make sure we don't get some
+    # amazingly flukey random number that breaks one of the tests
 
     with NumpyRNGContext(12345):
 
@@ -95,15 +95,29 @@ def test_sigma_clipped_stats():
     data = [0, 1]
     mask = np.array([True, False])
     result = sigma_clipped_stats(data, mask=mask)
-    assert result[0] == 1.
-    assert result[1] == 1.
-    assert result[2] == 0.
+    # Check that the result of np.ma.median was converted to a scalar
+    assert isinstance(result[1], float)
+    assert result == (1., 1., 0.)
 
     # test list data with mask_value
-    result2 = sigma_clipped_stats(data, mask_value=0.)
-    assert result2[0] == 1.
-    assert result2[1] == 1.
-    assert result2[2] == 0.
+    result = sigma_clipped_stats(data, mask_value=0.)
+    assert isinstance(result[1], float)
+    assert result == (1., 1., 0.)
+
+    # test without mask
+    data = [0, 2]
+    result = sigma_clipped_stats(data)
+    assert isinstance(result[1], float)
+    assert result == (1., 1., 1.)
+
+    _data = np.arange(10)
+    data = np.ma.MaskedArray([_data, _data, 10 * _data])
+    mean = sigma_clip(data, axis=0, sigma=1).mean(axis=0)
+    assert_equal(mean, _data)
+    mean, median, stddev = sigma_clipped_stats(data, axis=0, sigma=1)
+    assert_equal(mean, _data)
+    assert_equal(median, _data)
+    assert_equal(stddev, np.zeros_like(_data))
 
 
 def test_invalid_sigma_clip():
@@ -116,10 +130,10 @@ def test_invalid_sigma_clip():
 
     result = sigma_clip(data)
 
-    # Pre #4051 if data contains any NaN or infs sigma_clip returns the mask
-    # containing `False` only or TypeError if data also contains a masked value.
+    # Pre #4051 if data contains any NaN or infs sigma_clip returns the
+    # mask containing `False` only or TypeError if data also contains a
+    # masked value.
 
     assert result.mask[2, 2] == True
     assert result.mask[3, 4] == True
     assert result.mask[1, 1] == True
-
