@@ -15,6 +15,13 @@ class Conf(_config.ConfigNamespace):
         'The template that determines the name of a column if it cannot be '
         'determined. Uses new-style (format method) string formatting.',
         aliases=['astropy.table.column.auto_colname'])
+    default_notebook_table_class = _config.ConfigItem(
+        'table-striped table-bordered table-condensed',
+        'The table class to be used in Jupyter notebooks when displaying '
+        'tables (and not overridden). See <http://getbootstrap.com/css/#tables '
+        'for a list of useful bootstrap classes.')
+
+
 conf = Conf()
 
 
