@@ -21,13 +21,8 @@ from .np_utils import fix_column_name
 
 # These "shims" provide __getitem__ implementations for Column and MaskedColumn
 from ._column_mixins import _ColumnGetitemShim, _MaskedColumnGetitemShim
-from ..config import ConfigAlias
 
 
-AUTO_COLNAME = ConfigAlias(
-    '0.4', 'AUTO_COLNAME', 'auto_colname',
-    'astropy.table.column', 'astropy.table')
-
 # Create a generic TableFormatter object for use by bare columns with no
 # parent table.
 FORMATTER = pprint.TableFormatter()
@@ -711,6 +706,20 @@ class Column(BaseColumn):
                                           copy=copy, copy_indices=copy_indices)
         return self
 
+    def __setattr__(self, item, value):
+        if not isinstance(self, MaskedColumn) and item == "mask":
+            raise AttributeError("cannot set mask value to a column in non-masked Table")
+        super(Column, self).__setattr__(item, value)
+
+        if item == 'unit' and issubclass(self.dtype.type, np.number):
+            try:
+                converted = self.parent_table._convert_col_for_table(self)
+            except AttributeError:  # Either no parent table or parent table is None
+                pass
+            else:
+                if converted is not self:
+                    self.parent_table.replace_column(self.name, converted)
+
     def _base_repr_(self, html=False):
         # If scalar then just convert to correct numpy type and use numpy repr
         if self.ndim == 0:
