@@ -116,7 +116,6 @@ def column_group_by(column, keys):
     grouped_column : Column object with groups attr set accordingly
     """
     from .table import Table
-    from .column import Column
 
     if isinstance(keys, Table):
         keys = keys.as_array()
@@ -183,7 +182,7 @@ class BaseGroups(object):
     def __getitem__(self, item):
         parent = self.parent
 
-        if isinstance(item, int):
+        if isinstance(item, (int, np.integer)):
             i0, i1 = self.indices[item], self.indices[item + 1]
             out = parent[i0:i1]
             out.groups._keys = parent.groups.keys[item]
