@@ -36,7 +36,7 @@ from copy import deepcopy
 import numpy as np
 
 from ..extern import six
-from .bst import BST, FastBST, FastRBT, MinValue, MaxValue
+from .bst import MinValue, MaxValue
 from .sorted_array import SortedArray
 from ..time import Time
 
@@ -194,7 +194,7 @@ class Index(object):
         ----------
         row_specifier : int, list, ndarray, or slice
         '''
-        if isinstance(row_specifier, int):
+        if isinstance(row_specifier, (int, np.integer)):
             # single row
             return (row_specifier,)
         elif isinstance(row_specifier, (list, np.ndarray)):
