@@ -12,13 +12,14 @@ from __future__ import (absolute_import, division, print_function,
 
 import itertools
 from datetime import datetime
+from collections import defaultdict
 
 import numpy as np
 
-from .. import units as u
+from .. import units as u, constants as const
 from .. import _erfa as erfa
 from ..units import UnitConversionError
-from ..utils.compat.numpycompat import NUMPY_LT_1_7
+from ..utils.decorators import lazyproperty
 from ..utils.compat.misc import override__dir__
 from ..utils.data_info import MixinInfo, data_info_factory
 from ..utils.compat.numpy import broadcast_to
@@ -28,19 +29,12 @@ from .formats import (TIME_FORMATS, TIME_DELTA_FORMATS,
                       TimeJD, TimeUnique, TimeAstropyTime, TimeDatetime)
 # Import TimeFromEpoch to avoid breaking code that followed the old example of
 # making a custom timescale in the documentation.
-from .formats import TimeFromEpoch
+from .formats import TimeFromEpoch  # pylint: disable=W0611
 
 
 __all__ = ['Time', 'TimeDelta', 'TIME_SCALES', 'TIME_DELTA_SCALES',
            'ScaleValueError', 'OperandTypeError']
 
-try:
-    # Not guaranteed available at setup time
-    from . import erfa_time
-except ImportError:
-    if not _ASTROPY_SETUP_:
-        raise
-
 
 TIME_SCALES = ('tai', 'tcb', 'tcg', 'tdb', 'tt', 'ut1', 'utc')
 MULTI_HOPS = {('tai', 'tcb'): ('tt', 'tdb'),
@@ -87,14 +81,14 @@ SCALE_OFFSETS = {('tt', 'tai'): None,
 # triple-level dictionary, yay!
 SIDEREAL_TIME_MODELS = {
     'mean': {
-        'IAU2006': {'function': erfa_time.gmst06, 'scales': ('ut1', 'tt')},
-        'IAU2000': {'function': erfa_time.gmst00, 'scales': ('ut1', 'tt')},
-        'IAU1982': {'function': erfa_time.gmst82, 'scales': ('ut1',)}},
+        'IAU2006': {'function': erfa.gmst06, 'scales': ('ut1', 'tt')},
+        'IAU2000': {'function': erfa.gmst00, 'scales': ('ut1', 'tt')},
+        'IAU1982': {'function': erfa.gmst82, 'scales': ('ut1',)}},
     'apparent': {
-        'IAU2006A': {'function': erfa_time.gst06a, 'scales': ('ut1', 'tt')},
-        'IAU2000A': {'function': erfa_time.gst00a, 'scales': ('ut1', 'tt')},
-        'IAU2000B': {'function': erfa_time.gst00b, 'scales': ('ut1',)},
-        'IAU1994': {'function': erfa_time.gst94, 'scales': ('ut1',)}}}
+        'IAU2006A': {'function': erfa.gst06a, 'scales': ('ut1', 'tt')},
+        'IAU2000A': {'function': erfa.gst00a, 'scales': ('ut1', 'tt')},
+        'IAU2000B': {'function': erfa.gst00b, 'scales': ('ut1',)},
+        'IAU1994': {'function': erfa.gst94, 'scales': ('ut1',)}}}
 
 
 class TimeInfo(MixinInfo):
@@ -116,6 +110,7 @@ class TimeInfo(MixinInfo):
     # When Time has mean, std, min, max methods:
     # funcs = [lambda x: getattr(x, stat)() for stat_name in MixinInfo._stats])
 
+
 class Time(object):
     """
     Represent and manipulate times and dates for astronomy.
@@ -161,10 +156,6 @@ class Time(object):
         Make a copy of the input values
     """
 
-    _precision = 3  # Precision when for seconds as floating point
-    _in_subfmt = '*'  # Select subformat for inputting string times
-    _out_subfmt = '*'  # Select subformat for outputting string times
-
     SCALES = TIME_SCALES
     """List of time scales"""
 
@@ -206,21 +197,23 @@ class Time(object):
         else:
             self.location = None
 
-        if precision is not None:
-            self.precision = precision
-        if in_subfmt is not None:
-            self.in_subfmt = in_subfmt
-        if out_subfmt is not None:
-            self.out_subfmt = out_subfmt
+        if isinstance(val, Time):
+            # Update _time formatting parameters if explicitly specified
+            if precision is not None:
+                self._time.precision = precision
+            if in_subfmt is not None:
+                self._time.in_subfmt = in_subfmt
+            if out_subfmt is not None:
+                self._time.out_subfmt = out_subfmt
 
-        if isinstance(val, self.__class__):
             if scale is not None:
                 self._set_scale(scale)
         else:
-            self._init_from_vals(val, val2, format, scale, copy)
+            self._init_from_vals(val, val2, format, scale, copy,
+                                 precision, in_subfmt, out_subfmt)
 
-        if self.location and (self.location.size > 1
-                              and self.location.shape != self.shape):
+        if self.location and (self.location.size > 1 and
+                              self.location.shape != self.shape):
             try:
                 # check the location can be broadcast to self's shape.
                 self.location = broadcast_to(self.location, self.shape,
@@ -232,12 +225,19 @@ class Time(object):
                                  'one for each time.'
                                  .format(self.location.shape, self.shape))
 
-    def _init_from_vals(self, val, val2, format, scale, copy):
+    def _init_from_vals(self, val, val2, format, scale, copy,
+                        precision=None, in_subfmt=None, out_subfmt=None):
         """
         Set the internal _format, scale, and _time attrs from user
         inputs.  This handles coercion into the correct shapes and
         some basic input validation.
         """
+        if precision is None:
+            precision = 3
+        if in_subfmt is None:
+            in_subfmt = '*'
+        if out_subfmt is None:
+            out_subfmt = '*'
 
         # Coerce val into an array
         val = _make_array(val, copy)
@@ -259,10 +259,12 @@ class Time(object):
                                                    sorted(self.SCALES)))
 
         # Parse / convert input values into internal jd1, jd2 based on format
-        self._time = self._get_time_fmt(val, val2, format, scale)
+        self._time = self._get_time_fmt(val, val2, format, scale,
+                                        precision, in_subfmt, out_subfmt)
         self._format = self._time.name
 
-    def _get_time_fmt(self, val, val2, format, scale):
+    def _get_time_fmt(self, val, val2, format, scale,
+                      precision, in_subfmt, out_subfmt):
         """
         Given the supplied val, val2, format and scale try to instantiate
         the corresponding TimeFormat class to convert the input values into
@@ -296,8 +298,7 @@ class Time(object):
 
         for format, FormatClass in formats:
             try:
-                return FormatClass(val, val2, scale, self.precision,
-                                   self.in_subfmt, self.out_subfmt)
+                return FormatClass(val, val2, scale, precision, in_subfmt, out_subfmt)
             except UnitConversionError:
                 raise
             except (ValueError, TypeError):
@@ -424,7 +425,7 @@ class Time(object):
                     args.append(get_dt(jd1, jd2))
                     break
 
-            conv_func = getattr(erfa_time, sys1 + '_' + sys2)
+            conv_func = getattr(erfa, sys1 + sys2)
             jd1, jd2 = conv_func(*args)
         self._time = self.FORMATS[self.format](jd1, jd2, scale, self.precision,
                                                self.in_subfmt, self.out_subfmt,
@@ -436,14 +437,15 @@ class Time(object):
         Decimal precision when outputting seconds as floating point (int
         value between 0 and 9 inclusive).
         """
-        return self._precision
+        return self._time.precision
 
     @precision.setter
     def precision(self, val):
         if not isinstance(val, int) or val < 0 or val > 9:
             raise ValueError('precision attribute must be an int between '
                              '0 and 9')
-        self._precision = val
+        self._time.precision = val
+        del self.cache
 
     @property
     def in_subfmt(self):
@@ -451,26 +453,28 @@ class Time(object):
         Unix wildcard pattern to select subformats for parsing string input
         times.
         """
-        return self._in_subfmt
+        return self._time.in_subfmt
 
     @in_subfmt.setter
     def in_subfmt(self, val):
         if not isinstance(val, six.string_types):
             raise ValueError('in_subfmt attribute must be a string')
-        self._in_subfmt = val
+        self._time.in_subfmt = val
+        del self.cache
 
     @property
     def out_subfmt(self):
         """
         Unix wildcard pattern to select subformats for outputting times.
         """
-        return self._out_subfmt
+        return self._time.out_subfmt
 
     @out_subfmt.setter
     def out_subfmt(self, val):
         if not isinstance(val, six.string_types):
             raise ValueError('out_subfmt attribute must be a string')
-        self._out_subfmt = val
+        self._time.out_subfmt = val
+        del self.cache
 
     @property
     def ndim(self):
@@ -536,7 +540,87 @@ class Time(object):
     @property
     def value(self):
         """Time value(s) in current format"""
-        return self._shaped_like_input(self._time.to_value(parent=self))
+        # The underlying way to get the time values for the current format is:
+        #     self._shaped_like_input(self._time.to_value(parent=self))
+        # This is done in __getattr__.  By calling getattr(self, self.format)
+        # the ``value`` attribute is cached.
+        return getattr(self, self.format)
+
+    def light_travel_time(self, skycoord, kind='barycentric', location=None):
+        """Light travel time correction to the barycentre or heliocentre.
+
+        The frame transformations used to calculate the location of the solar
+        system barycentre and the heliocentre rely on the erfa routine epv00,
+        which is consistent with the JPL DE405 ephemeris to an accuracy of
+        11.2 km, corresponding to a light travel time of 4 microseconds.
+
+        The routine assumes the source(s) are at large distance, i.e., neglects
+        finite-distance effects.
+
+        Parameters
+        ----------
+        skycoord: `~astropy.coordinates.SkyCoord`
+            The sky location to calculate the correction for.
+        kind: str, optional
+            ``'barycentric'`` (default) or ``'heliocentric'``
+        location: `~astropy.coordinates.EarthLocation`, optional
+            The location of the observatory to calculate the correction for.
+            If no location is given, the ``location`` attribute of the Time
+            object is used
+
+        Returns
+        -------
+        time_offset: `~astropy.time.TimeDelta`
+            The time offset between the barycentre or Heliocentre and Earth,
+            in TDB seconds.  Should be added to the original time to get the
+            time in the Solar system barycentre or the Heliocentre.
+        """
+
+        if kind.lower() not in ('barycentric', 'heliocentric'):
+            raise ValueError("'kind' parameter must be one of 'heliocentric' "
+                             "or 'barycentric'")
+
+        if location is None:
+            if self.location is None:
+                raise ValueError('An EarthLocation needs to be set or passed '
+                                 'in to calculate bary- or heliocentric '
+                                 'corrections')
+            location = self.location
+
+        from ..coordinates import (UnitSphericalRepresentation, CartesianRepresentation,
+                                   HCRS, ICRS, GCRS, EarthLocation, SkyCoord)
+
+        # ensure sky location is ICRS compatible
+        if not skycoord.is_transformable_to(ICRS()):
+            raise ValueError("Given skycoord is not transformable to the ICRS")
+
+        # get location of observatory in ITRS coordinates at this Time
+        try:
+            itrs = location.get_itrs(obstime=self)
+        except:
+            raise ValueError("Supplied location does not have a valid `get_itrs` method")
+
+        if kind.lower() == 'heliocentric':
+            # convert to heliocentric coordinates, aligned with ICRS
+            cpos = itrs.transform_to(HCRS(obstime=self)).cartesian.xyz
+        else:
+            # first we need to convert to GCRS coordinates with the correct
+            # obstime, since ICRS coordinates have no frame time
+            gcrs_coo = itrs.transform_to(GCRS(obstime=self))
+            # convert to barycentric (BCRS) coordinates, aligned with ICRS
+            cpos = gcrs_coo.transform_to(ICRS()).cartesian.xyz
+
+        # get unit ICRS vector to star
+        spos = (skycoord.icrs.represent_as(UnitSphericalRepresentation).
+                represent_as(CartesianRepresentation).xyz)
+
+        # Move X,Y,Z to last dimension, to enable possible broadcasting below.
+        cpos = np.rollaxis(cpos, 0, cpos.ndim)
+        spos = np.rollaxis(spos, 0, spos.ndim)
+
+        # calculate light travel time correction
+        tcor_val = (spos * cpos).sum(axis=-1) / const.c
+        return TimeDelta(tcor_val, scale='tdb')
 
     def sidereal_time(self, kind, longitude=None, model=None):
         """Calculate sidereal time.
@@ -676,11 +760,10 @@ class Time(object):
         # To avoid recalculating integer day + fraction, no longer just
         # instantiate a new class instance, but rather do the steps by hand.
         # This also avoids quite a bit of unnecessary work in __init__
-        ###  tm = self.__class__(self._time.jd1, self._time.jd2,
-        ###                      format='jd', scale=self.scale, copy=copy)
+        #  tm = self.__class__(self._time.jd1, self._time.jd2,
+        #                      format='jd', scale=self.scale, copy=copy)
         return self._replicate(format=format, method='copy' if copy else None)
 
-
     def _replicate(self, method=None, *args, **kwargs):
         """Replicate a time object, possibly applying a method to the arrays.
 
@@ -926,12 +1009,7 @@ class Time(object):
         """
         # first get the minimum at normal precision.
         jd = self.jd1 + self.jd2
-        if NUMPY_LT_1_7:
-            approx = jd.min(axis)
-            if axis is not None:
-                approx = np.expand_dims(approx, axis)
-        else:
-            approx = jd.min(axis, keepdims=True)
+        approx = jd.min(axis, keepdims=True)
 
         # Approx is very close to the true minimum, and by subtracting it at
         # full precision, all numbers near 0 can be represented correctly,
@@ -953,12 +1031,7 @@ class Time(object):
         """
         # For procedure, see comment on argmin.
         jd = self.jd1 + self.jd2
-        if NUMPY_LT_1_7:
-            approx = jd.max(axis)
-            if axis is not None:
-                approx = np.expand_dims(approx, axis)
-        else:
-            approx = jd.max(axis, keepdims=True)
+        approx = jd.max(axis, keepdims=True)
 
         dt = (self.jd1 - approx) + self.jd2
         return dt.argmax(axis, out)
@@ -1044,18 +1117,38 @@ class Time(object):
         return self[self._advanced_index(self.argsort(axis), axis,
                                          keepdims=True)]
 
+    @lazyproperty
+    def cache(self):
+        """
+        Return the cache associated with this instance.
+        """
+        return defaultdict(dict)
+
     def __getattr__(self, attr):
         """
         Get dynamic attributes to output format or do timescale conversion.
         """
         if attr in self.SCALES and self.scale is not None:
-            tm = self.replicate()
-            tm._set_scale(attr)
-            return tm
+            cache = self.cache['scale']
+            if attr not in cache:
+                if attr == self.scale:
+                    tm = self
+                else:
+                    tm = self.replicate()
+                    tm._set_scale(attr)
+                cache[attr] = tm
+            return cache[attr]
 
         elif attr in self.FORMATS:
-            tm = self.replicate(format=attr)
-            return tm.value
+            cache = self.cache['format']
+            if attr not in cache:
+                if attr == self.format:
+                    tm = self
+                else:
+                    tm = self.replicate(format=attr)
+                value = tm._shaped_like_input(tm._time.to_value(parent=tm))
+                cache[attr] = value
+            return cache[attr]
 
         elif attr in TIME_SCALES:  # allowed ones done above (self.SCALES)
             if self.scale is None:
@@ -1106,7 +1199,7 @@ class Time(object):
             ``astropy.utils.iers``)
         return_status : bool
             Whether to return status values.  If `False` (default), iers
-            raises `~.exceptions.IndexError` if any time is out of the range
+            raises `IndexError` if any time is out of the range
             covered by the IERS table.
 
         Returns
@@ -1135,22 +1228,6 @@ class Time(object):
             >>> delta, status = t.get_delta_ut1_utc(return_status=True)
             >>> status == TIME_BEFORE_IERS_RANGE
             array([ True, False], dtype=bool)
-
-        To use an updated IERS A bulletin to calculate UT1-UTC
-        (see also ``astropy.utils.iers``)::
-
-            >>> from astropy.utils.iers import IERS_A, IERS_A_URL
-            >>> from astropy.utils.data import download_file
-            >>> t = Time(['1974-01-01', '2000-01-01'], scale='utc')
-            >>> iers_a_file = download_file(IERS_A_URL,
-            ...                             cache=True)        # doctest: +REMOTE_DATA
-            Downloading ... [Done]
-            >>> iers_a = IERS_A.open(iers_a_file)              # doctest: +REMOTE_DATA
-            >>> t.delta_ut1_utc = t.get_delta_ut1_utc(iers_a)  # doctest: +REMOTE_DATA
-
-        The delta_ut1_utc property will be used to calculate t.ut1;
-        raises IndexError if any of the times is out of range.
-
         """
         if iers_table is None:
             from ..utils.iers import IERS
@@ -1169,8 +1246,8 @@ class Time(object):
         # Sec. 4.3.1: the arg DUT is the quantity delta_UT1 = UT1 - UTC in
         # seconds. It is obtained from tables published by the IERS.
         if not hasattr(self, '_delta_ut1_utc'):
-            from ..utils.iers import IERS
-            iers_table = IERS.open()
+            from ..utils.iers import IERS_Auto
+            iers_table = IERS_Auto.open()
             # jd1, jd2 are normally set (see above), except if delta_ut1_utc
             # is access directly; ensure we behave as expected for that case
             if jd1 is None:
@@ -1186,7 +1263,7 @@ class Time(object):
             if scale == 'ut1':
                 # calculate UTC using the offset we got; the ERFA routine
                 # is tolerant of leap seconds, so will do this right
-                jd1_utc, jd2_utc = erfa_time.ut1_utc(jd1, jd2, delta)
+                jd1_utc, jd2_utc = erfa.ut1utc(jd1, jd2, delta)
                 # calculate a better estimate using the nearly correct UTC
                 delta = iers_table.ut1_utc(jd1_utc, jd2_utc)
 
@@ -1199,6 +1276,7 @@ class Time(object):
         if hasattr(val, 'to'):  # Matches Quantity but also TimeDelta.
             val = val.to(u.second).value
         self._delta_ut1_utc = val
+        del self.cache
 
     # Note can't use @property because _get_delta_tdb_tt is explicitly
     # called with the optional jd1 and jd2 args.
@@ -1224,8 +1302,8 @@ class Time(object):
             # TDB or TT) to an approximate UT1.  Since TT and TDB are
             # pretty close (few msec?), assume TT.  Similarly, since the
             # UT1 terms are very small, use UTC instead of UT1.
-            njd1, njd2 = erfa_time.tt_tai(jd1, jd2)
-            njd1, njd2 = erfa_time.tai_utc(njd1, njd2)
+            njd1, njd2 = erfa.tttai(jd1, jd2)
+            njd1, njd2 = erfa.taiutc(njd1, njd2)
             # subtract 0.5, so UT is fraction of the day from midnight
             ut = day_frac(njd1 - 0.5, njd2)[1]
 
@@ -1238,7 +1316,7 @@ class Time(object):
             lon = location.longitude
             rxy = np.hypot(location.x, location.y)
             z = location.z
-            self._delta_tdb_tt = erfa_time.d_tdb_tt(
+            self._delta_tdb_tt = erfa.dtdb(
                 jd1, jd2, ut, lon.to(u.radian).value,
                 rxy.to(u.km).value, z.to(u.km).value)
 
@@ -1249,6 +1327,7 @@ class Time(object):
         if hasattr(val, 'to'):  # Matches Quantity but also TimeDelta.
             val = val.to(u.second).value
         self._delta_tdb_tt = val
+        del self.cache
 
     # Note can't use @property because _get_delta_tdb_tt is explicitly
     # called with the optional jd1 and jd2 args.
@@ -1276,13 +1355,13 @@ class Time(object):
         # we need a constant scale to calculate, which is guaranteed for
         # TimeDelta, but not for Time (which can be UTC)
         if other_is_delta:  # T - Tdelta
+            out = self.replicate()
             if self.scale in other.SCALES:
-                out = self.replicate()
                 if other.scale not in (out.scale, None):
                     other = getattr(other, out.scale)
             else:
-                out = getattr(self, (other.scale if other.scale is not None
-                                     else 'tai'))
+                out._set_scale(other.scale if other.scale is not None
+                               else 'tai')
             # remove attributes that are invalidated by changing time
             for attr in ('_delta_ut1_utc', '_delta_tdb_tt'):
                 if hasattr(out, attr):
@@ -1303,8 +1382,9 @@ class Time(object):
 
         out._time.jd1, out._time.jd2 = day_frac(jd1, jd2)
 
-        if other_is_delta and out.scale != self.scale:
-            return getattr(out, self.scale)
+        if other_is_delta:
+            # Go back to left-side scale if needed
+            out._set_scale(self.scale)
 
         return out
 
@@ -1325,13 +1405,12 @@ class Time(object):
         # ideally, we calculate in the scale of the Time item, since that is
         # what we want the output in, but this may not be possible, since
         # TimeDelta cannot be converted arbitrarily
+        out = self.replicate()
         if self.scale in other.SCALES:
-            out = self.replicate()
             if other.scale not in (out.scale, None):
                 other = getattr(other, out.scale)
         else:
-            out = getattr(self, (other.scale if other.scale is not None
-                                 else 'tai'))
+            out._set_scale(other.scale if other.scale is not None else 'tai')
 
         # remove attributes that are invalidated by changing time
         for attr in ('_delta_ut1_utc', '_delta_tdb_tt'):
@@ -1343,8 +1422,8 @@ class Time(object):
 
         out._time.jd1, out._time.jd2 = day_frac(jd1, jd2)
 
-        if out.scale != self.scale:
-            return getattr(out, self.scale)
+        # Go back to left-side scale if needed
+        out._set_scale(self.scale)
 
         return out
 
@@ -1418,7 +1497,6 @@ class Time(object):
     to_datetime.__doc__ = TimeDatetime.to_value.__doc__
 
 
-
 class TimeDelta(Time):
     """
     Represent the time difference between two times.
@@ -1466,7 +1544,7 @@ class TimeDelta(Time):
     """Dict of time delta formats."""
 
     def __init__(self, val, val2=None, format=None, scale=None, copy=False):
-        if isinstance(val, self.__class__):
+        if isinstance(val, TimeDelta):
             if scale is not None:
                 self._set_scale(scale)
         else:
