@@ -6,16 +6,18 @@ import copy
 import functools
 import sys
 import datetime
+from copy import deepcopy
 
 import numpy as np
 
 from ...tests.helper import pytest, catch_warnings
+from ...tests.disable_internet import INTERNET_OFF
 from ...extern import six
 from ...utils import isiterable
-from .. import (Time, ScaleValueError, erfa_time, TIME_SCALES, TimeString,
-                TimezoneInfo)
+from .. import Time, ScaleValueError, TIME_SCALES, TimeString, TimezoneInfo
 from ...coordinates import EarthLocation
 from ... import units as u
+from ... import _erfa as erfa
 try:
     import pytz
     HAS_PYTZ = True
@@ -30,8 +32,14 @@ allclose_sec = functools.partial(np.allclose, rtol=2. ** -52,
 allclose_year = functools.partial(np.allclose, rtol=2. ** -52,
                                   atol=0.)  # 14 microsec at current epoch
 
+def setup_function(func):
+    func.FORMATS_ORIG = deepcopy(Time.FORMATS)
 
 
+def teardown_function(func):
+    Time.FORMATS.clear()
+    Time.FORMATS.update(func.FORMATS_ORIG)
+
 
 class TestBasic():
     """Basic tests stemming from initial example and API reference"""
@@ -217,26 +225,14 @@ class TestBasic():
         # Uses initial class-defined precision=3
         assert t.iso == '2010-01-01 00:00:00.000'
 
-        # Set global precision = 5  XXX this uses private var, FIX THIS
-        Time._precision = 5
-        assert t.iso == '2010-01-01 00:00:00.00000'
-
         # Set instance precision to 9
         t.precision = 9
         assert t.iso == '2010-01-01 00:00:00.000000000'
         assert t.tai.utc.iso == '2010-01-01 00:00:00.000000000'
 
-        # Restore global to original default of 3, instance is still at 9
-        Time._precision = 3
-        assert t.iso == '2010-01-01 00:00:00.000000000'
-
-        # Make a new time instance and confirm precision = 3
-        t = Time('2010-01-01 00:00:00', format='iso', scale='utc')
-        assert t.iso == '2010-01-01 00:00:00.000'
-
     def test_transforms(self):
         """Transform from UTC to all supported time scales (TAI, TCB, TCG,
-        TDB, TT, UT1, UTC).  This requires auxilliary information (latitude and
+        TDB, TT, UT1, UTC).  This requires auxiliary information (latitude and
         longitude)."""
 
         lat = 19.48125
@@ -273,7 +269,7 @@ class TestBasic():
 
     def test_location_array(self):
         """Check that location arrays are checked for size and used
-        for the corresponding times.  Also checks that erfa_time.d_tdb_tt
+        for the corresponding times.  Also checks that erfa
         can handle array-valued locations, and can broadcast these if needed.
         """
 
@@ -702,16 +698,16 @@ class TestSofaErrors():
         im = np.array([2000], dtype=np.intc)  # bad month
         id = np.array([2000], dtype=np.intc)  # bad day
         with pytest.raises(ValueError):  # bad month, fatal error
-            djm0, djm = erfa_time.cal2jd(iy, im, id)
+            djm0, djm = erfa.cal2jd(iy, im, id)
 
         iy[0] = -5000
         im[0] = 2
         with pytest.raises(ValueError):  # bad year, fatal error
-            djm0, djm = erfa_time.cal2jd(iy, im, id)
+            djm0, djm = erfa.cal2jd(iy, im, id)
 
         iy[0] = 2000
         with catch_warnings() as w:
-            djm0, djm = erfa_time.cal2jd(iy, im, id)
+            djm0, djm = erfa.cal2jd(iy, im, id)
         assert len(w) == 1
         assert 'bad day    (JD computed)' in six.text_type(w[0].message)
 
@@ -750,6 +746,12 @@ class TestCopyReplicate():
         # This is not allowed publicly, but here we hack the internal time
         # and location values to show that t and t2 are sharing references.
         t2._time.jd1 += 100.0
+
+        # Need to delete the cached yday attributes (only an issue because
+        # of the internal _time hack).
+        del t.cache
+        del t2.cache
+
         assert t.yday == t2.yday
         assert t.yday != t_yday  # prove that it changed
         t2_loc_x_view = t2.location.x
@@ -769,6 +771,12 @@ class TestCopyReplicate():
         # This is not allowed publicly, but here we hack the internal time
         # and location values to show that t and t2 are not sharing references.
         t2._time.jd1 += 100.0
+
+        # Need to delete the cached yday attributes (only an issue because
+        # of the internal _time hack).
+        del t.cache
+        del t2.cache
+
         assert t.yday != t2.yday
         assert t.yday == t_yday  # prove that it did not change
         t2_loc_x_view = t2.location.x
@@ -888,8 +896,16 @@ def test_TimeFormat_scale():
     assert t.unix == t.utc.unix
 
 def test_scale_conversion():
-    with pytest.raises(ScaleValueError):
-        t = Time(Time.now().cxcsec, format='cxcsec', scale='ut1')
+    if INTERNET_OFF:
+        # With internet off (which is the default for testing) then this will
+        # fall back to the bundled IERS-B table and raise an exception.  But when
+        # testing with --remote-data the IERS_Auto class will get the latest IERS-A
+        # and this works.
+        with pytest.raises(ScaleValueError):
+            Time(Time.now().cxcsec, format='cxcsec', scale='ut1')
+    else:
+        Time(Time.now().cxcsec, format='cxcsec', scale='ut1')
+
 
 
 def test_byteorder():
@@ -1051,3 +1067,28 @@ def test_to_datetime_pytz():
     for dt, tz_dt in zip(time.datetime, tz_aware_datetime):
         assert tz.tzname(dt) == tz_dt.tzname()
     assert np.all(time == forced_to_astropy_time)
+
+def test_cache():
+    t = Time('2010-09-03 00:00:00')
+    t2 = Time('2010-09-03 00:00:00')
+
+    # Time starts out without a cache
+    assert 'cache' not in t.__dict__
+
+    # Access the iso format and confirm that the cached version is as expected
+    t.iso
+    assert t.cache['format']['iso'] == t2.iso
+
+    # Access the TAI scale and confirm that the cached version is as expected
+    t.tai
+    assert t.cache['scale']['tai'] == t2.tai
+
+    # New Time object after scale transform does not have a cache yet
+    assert 'cache' not in t.tt.__dict__
+
+    # Clear the cache
+    del t.cache
+    assert 'cache' not in t.__dict__
+    # Check accessing the cache creates an empty dictionary
+    assert not t.cache
+    assert 'cache' in t.__dict__
