@@ -11,20 +11,19 @@ from __future__ import (absolute_import, unicode_literals, division,
 
 # Standard library
 import numbers
+from fractions import Fraction
 
 import numpy as np
 
 # AstroPy
 from ..extern import six
 from .core import (Unit, dimensionless_unscaled, UnitBase, UnitsError,
-                   get_current_unit_registry)
+                   get_current_unit_registry, UnitConversionError)
 from .format.latex import Latex
-from ..utils.compat import NUMPY_LT_1_7, NUMPY_LT_1_8, NUMPY_LT_1_9
-from ..utils.compat.fractions import Fraction
+from ..utils.compat import NUMPY_LT_1_8, NUMPY_LT_1_9
 from ..utils.compat.misc import override__dir__
 from ..utils.misc import isiterable, InheritDocstrings
 from ..utils.data_info import ParentDtypeInfo
-from .utils import validate_power
 from .. import config as _config
 
 
@@ -357,17 +356,27 @@ class Quantity(np.ndarray):
         # amount that depends on one of the input values, so we need to treat
         # this as a special case.
         # TODO: find a better way to deal with this case
-        if function is np.power and result_unit is not dimensionless_unscaled:
+        if function is np.power and result_unit != dimensionless_unscaled:
 
             if units[1] is None:
                 p = args[1]
             else:
                 p = args[1].to(dimensionless_unscaled).value
 
-            result_unit = result_unit ** validate_power(p)
+            try:
+                result_unit = result_unit ** p
+            except ValueError as exc:
+                # Changing the unit does not work for, e.g., array-shaped
+                # power, but this is OK if we're (scaled) dimensionless.
+                try:
+                    converters[0] = units[0]._get_converter(
+                        dimensionless_unscaled)
+                except UnitConversionError:
+                    raise exc
+                else:
+                    result_unit = dimensionless_unscaled
 
         # We now prepare the output object
-
         if self is obj:
 
             # this happens if the output object is self, which happens
@@ -495,10 +504,20 @@ class Quantity(np.ndarray):
                 # For output arrays that require scaling, we can reuse the
                 # output array to perform the scaling in place, as long as the
                 # array is not integral. Here, we set the obj_array to `None`
-                # when it can not be used to store the scaled result.
-                if not (result_unit is None or
-                        np.can_cast(np.result_type(*inputs), obj_array.dtype)):
+                # when it cannot be used to store the scaled result.
+                # Use a try/except, since np.result_type can fail, which would
+                # break the wrapping #4770.
+                try:
+                    tmp_dtype = np.result_type(*inputs)
+                # Catch the appropriate exceptions: TypeError or ValueError in
+                # case the result_type raised an Exception, i.e. inputs is list
+                except (TypeError, ValueError):
                     obj_array = None
+                else:
+                    # Explicitly check if it can store the result.
+                    if not (result_unit is None or
+                            np.can_cast(tmp_dtype, obj_array.dtype)):
+                        obj_array = None
 
                 # Re-compute the output using the ufunc
                 if function.nin == 1:
@@ -1008,31 +1027,24 @@ class Quantity(np.ndarray):
         lstr
             A LaTeX string with the contents of this Quantity
         """
-        if NUMPY_LT_1_7:
-            if self.isscalar:
-                latex_value = Latex.format_exponential_notation(self.value)
-            else:
-                raise NotImplementedError('Cannot represent Quantity arrays '
-                                          'in LaTex format for numpy < v1.7.')
-        else:
-            # need to do try/finally because "threshold" cannot be overridden
-            # with array2string
-            pops = np.get_printoptions()
-            try:
-                formatter = {'all' : Latex.format_exponential_notation,
-                             'str_kind': lambda x: x}
-                if conf.latex_array_threshold > -1:
-                    np.set_printoptions(threshold=conf.latex_array_threshold,
-                                        formatter=formatter)
+        # need to do try/finally because "threshold" cannot be overridden
+        # with array2string
+        pops = np.get_printoptions()
+        try:
+            formatter = {'float_kind': Latex.format_exponential_notation}
+            if conf.latex_array_threshold > -1:
+                np.set_printoptions(threshold=conf.latex_array_threshold,
+                                    formatter=formatter)
 
-                # the view is needed for the scalar case - value might be float
-                latex_value = np.array2string(self.view(np.ndarray),
-                                              style=Latex.format_exponential_notation,
-                                              max_line_width=np.inf,
-                                              separator=',~')
-                latex_value = latex_value.replace('...', r'\dots')
-            finally:
-                np.set_printoptions(**pops)
+            # the view is needed for the scalar case - value might be float
+            latex_value = np.array2string(
+                self.view(np.ndarray),
+                style=(Latex.format_exponential_notation
+                       if self.dtype.kind == 'f' else repr),
+                max_line_width=np.inf, separator=',~')
+            latex_value = latex_value.replace('...', r'\dots')
+        finally:
+            np.set_printoptions(**pops)
 
         # Format unit
         # [1:-1] strips the '$' on either side needed for math mode
@@ -1315,49 +1327,26 @@ class Quantity(np.ndarray):
     def round(self, decimals=0, out=None):
         return self._wrap_function(np.round, decimals, out=out)
 
-    if NUMPY_LT_1_7:
-        # 'keepdims' was not yet available.
-        def max(self, axis=None, out=None):
-            return self._wrap_function(np.max, axis, out=out)
+    def max(self, axis=None, out=None, keepdims=False):
+        return self._wrap_function(np.max, axis, out=out, keepdims=keepdims)
 
-        def min(self, axis=None, out=None):
-            return self._wrap_function(np.min, axis, out=out)
+    def min(self, axis=None, out=None, keepdims=False):
+        return self._wrap_function(np.min, axis, out=out, keepdims=keepdims)
 
-        def sum(self, axis=None, dtype=None, out=None):
-            return self._wrap_function(np.sum, axis, dtype, out=out)
+    def sum(self, axis=None, dtype=None, out=None, keepdims=False):
+        return self._wrap_function(np.sum, axis, dtype, out=out,
+                                   keepdims=keepdims)
 
-        def prod(self, axis=None, dtype=None, out=None):
-            if not self.unit.is_unity():
-                raise ValueError("cannot use prod on scaled or "
-                                 "non-dimensionless Quantity arrays")
-            return self._wrap_function(np.prod, axis, dtype, out=out)
+    def prod(self, axis=None, dtype=None, out=None, keepdims=False):
+        if not self.unit.is_unity():
+            raise ValueError("cannot use prod on scaled or "
+                             "non-dimensionless Quantity arrays")
+        return self._wrap_function(np.prod, axis, dtype, out=out,
+                                   keepdims=keepdims)
 
-        # 'out' was not yet available.
-        def dot(self, b):
-            result_unit = self.unit * getattr(b, 'unit', dimensionless_unscaled)
-            return self._wrap_function(np.dot, b, unit=result_unit)
-
-    else:
-        def max(self, axis=None, out=None, keepdims=False):
-            return self._wrap_function(np.max, axis, out=out, keepdims=keepdims)
-
-        def min(self, axis=None, out=None, keepdims=False):
-            return self._wrap_function(np.min, axis, out=out, keepdims=keepdims)
-
-        def sum(self, axis=None, dtype=None, out=None, keepdims=False):
-            return self._wrap_function(np.sum, axis, dtype, out=out,
-                                       keepdims=keepdims)
-
-        def prod(self, axis=None, dtype=None, out=None, keepdims=False):
-            if not self.unit.is_unity():
-                raise ValueError("cannot use prod on scaled or "
-                                 "non-dimensionless Quantity arrays")
-            return self._wrap_function(np.prod, axis, dtype, out=out,
-                                       keepdims=keepdims)
-
-        def dot(self, b, out=None):
-            result_unit = self.unit * getattr(b, 'unit', dimensionless_unscaled)
-            return self._wrap_function(np.dot, b, out=out, unit=result_unit)
+    def dot(self, b, out=None):
+        result_unit = self.unit * getattr(b, 'unit', dimensionless_unscaled)
+        return self._wrap_function(np.dot, b, out=out, unit=result_unit)
 
     def cumsum(self, axis=None, dtype=None, out=None):
         return self._wrap_function(np.cumsum, axis, dtype, out=out)
@@ -1368,7 +1357,6 @@ class Quantity(np.ndarray):
                              "non-dimensionless Quantity arrays")
         return self._wrap_function(np.cumprod, axis, dtype, out=out)
 
-
     # Calculation: override methods that do not make sense.
 
     def all(self, axis=None, out=None):
