@@ -12,8 +12,8 @@ import itertools
 import numpy as np
 from numpy.testing.utils import assert_allclose
 
-from ...tests.helper import pytest
-from ... import units as u
+from ...tests.helper import pytest, assert_quantity_allclose
+from ... import units as u, constants as c
 
 lu_units = [u.dex, u.mag, u.decibel]
 
@@ -71,6 +71,32 @@ class TestLogUnitCreation(object):
             lu_cls(physical_unit, u.m)
 
 
+def test_predefined_magnitudes():
+    assert_quantity_allclose((-21.1*u.STmag).physical,
+                             1.*u.erg/u.cm**2/u.s/u.AA)
+    assert_quantity_allclose((-48.6*u.ABmag).physical,
+                             1.*u.erg/u.cm**2/u.s/u.Hz)
+    assert_quantity_allclose((0*u.M_bol).physical, c.L_bol0)
+    assert_quantity_allclose((0*u.m_bol).physical,
+                             c.L_bol0/(4.*np.pi*(10.*c.pc)**2))
+
+
+def test_predefined_reinitialisation():
+    assert u.mag('ST') == u.STmag
+    assert u.mag('AB') == u.ABmag
+    assert u.mag('Bol') == u.M_bol
+    assert u.mag('bol') == u.m_bol
+
+
+def test_predefined_string_roundtrip():
+    """Ensure roundtripping; see #5015"""
+    with u.magnitude_zero_points.enable():
+        assert u.Unit(u.STmag.to_string()) == u.STmag
+        assert u.Unit(u.ABmag.to_string()) == u.ABmag
+        assert u.Unit(u.M_bol.to_string()) == u.M_bol
+        assert u.Unit(u.m_bol.to_string()) == u.m_bol
+
+
 class TestLogUnitStrings(object):
 
     def test_str(self):
@@ -92,6 +118,12 @@ class TestLogUnitStrings(object):
         assert repr(lu3) == 'MagUnit("Jy", unit="2 mag")'
         assert lu3.to_string() == '2 mag(Jy)'
 
+        lu4 = u.mag(u.ct)
+        assert lu4.to_string('generic') == 'mag(ct)'
+        assert lu4.to_string('latex') == ('$\\mathrm{mag}$$\\mathrm{\\left( '
+                                          '\\mathrm{ct} \\right)}$')
+
+
 class TestLogUnitConversion(object):
     @pytest.mark.parametrize('lu_unit, physical_unit',
                              itertools.product(lu_units, pu_sample))
@@ -129,7 +161,7 @@ class TestLogUnitConversion(object):
     def test_container_unit_conversion(self, lu_unit):
         """Check that conversion to logarithmic units (u.mag, u.dB, u.dex)
         is only possible when the physical unit is dimensionless."""
-        values = np.linspace(0.,10.,6.)
+        values = np.linspace(0., 10., 6)
         lu1 = lu_unit(u.dimensionless_unscaled)
         assert lu1.is_equivalent(lu1.function_unit)
         assert_allclose(lu1.to(lu1.function_unit, values), values)
@@ -145,7 +177,7 @@ class TestLogUnitConversion(object):
     def test_subclass_conversion(self, flu_unit, tlu_unit, physical_unit):
         """Check various LogUnit subclasses are equivalent and convertible
         to each other if they correspond to equivalent physical units."""
-        values = np.linspace(0.,10.,6.)
+        values = np.linspace(0., 10., 6)
         flu = flu_unit(physical_unit)
 
         tlu = tlu_unit(physical_unit)
