@@ -1,10 +1,7 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from astropy.extern import six
-
 from ... import units as u
-from ...extern import six
 from ...tests.helper import pytest
 
 
@@ -70,7 +67,7 @@ def test_wrong_unit():
 
     with pytest.raises(u.UnitsError) as e:
         solarx, solary = myfunc_args(1*u.arcsec, 100*u.km)
-    assert str(e.value) == "Argument 'solary' to function 'myfunc_args' must be in units convertable to 'deg'."
+    assert str(e.value) == "Argument 'solary' to function 'myfunc_args' must be in units convertible to 'deg'."
 
 def test_not_quantity():
     @u.quantity_input(solarx=u.arcsec, solary=u.deg)
@@ -129,7 +126,7 @@ def test_kwarg_wrong_unit():
 
     with pytest.raises(u.UnitsError) as e:
         solarx, solary = myfunc_args(1*u.arcsec, solary=100*u.km)
-    assert str(e.value) == "Argument 'solary' to function 'myfunc_args' must be in units convertable to 'deg'."
+    assert str(e.value) == "Argument 'solary' to function 'myfunc_args' must be in units convertible to 'deg'."
 
 def test_kwarg_not_quantity():
     @u.quantity_input(solarx=u.arcsec, solary=u.deg)
@@ -192,4 +189,3 @@ def test_kwargs_extra():
     assert isinstance(solarx, u.Quantity)
 
     assert solarx.unit == u.deg
-
