@@ -15,23 +15,24 @@ import difflib
 import inspect
 import json
 import os
-import re
 import signal
 import sys
 import traceback
 import unicodedata
+import locale
+import threading
 
-from collections import defaultdict
+from contextlib import contextmanager
+from collections import defaultdict, OrderedDict
 
 from ..extern import six
 from ..extern.six.moves import urllib
-from ..utils.compat.odict import OrderedDict
 
 
 __all__ = ['isiterable', 'silence', 'format_exception', 'NumpyRNGContext',
            'find_api_page', 'is_path_hidden', 'walk_skip_hidden',
            'JsonCustomEncoder', 'indent', 'InheritDocstrings',
-           'OrderedDescriptor', 'OrderedDescriptorContainer']
+           'OrderedDescriptor', 'OrderedDescriptorContainer', 'set_locale']
 
 
 def isiterable(obj):
@@ -248,7 +249,7 @@ def find_api_page(obj, version=None, openinbrowser=True, timeout=None):
         ivers, proj, vers, compr  = headerlines
         if not 'The remainder of this file is compressed using zlib' in compr:
             raise ValueError('The file downloaded from {0} does not seem to be'
-                             'the usal Sphinx objects.inv format.  Maybe it '
+                             'the usual Sphinx objects.inv format.  Maybe it '
                              'has changed?'.format(baseurl + 'objects.inv'))
 
         compressed = oiread[(idx+1):]
@@ -813,3 +814,39 @@ class OrderedDescriptorContainer(type):
 
         super(OrderedDescriptorContainer, cls).__init__(cls_name, bases,
                                                         members)
+
+
+LOCALE_LOCK = threading.Lock()
+
+@contextmanager
+def set_locale(name):
+    """
+    Context manager to temporarily set the locale to ``name``.
+
+    An example is setting locale to "C" so that the C strtod()
+    function will use "." as the decimal point to enable consistent
+    numerical string parsing.
+
+    Note that one cannot nest multiple set_locale() context manager
+    statements as this causes a threading lock.
+
+    This code taken from http://stackoverflow.com/questions/18593661/.
+
+    Parameters
+    ==========
+    name : str
+        Locale name, e.g. "C" or "fr_FR".
+    """
+    name = str(name)
+
+    with LOCALE_LOCK:
+        saved = locale.setlocale(locale.LC_ALL)
+        if saved == name:
+            # Don't do anything if locale is already the requested locale
+            yield
+        else:
+            try:
+                locale.setlocale(locale.LC_ALL, name)
+                yield
+            finally:
+                locale.setlocale(locale.LC_ALL, saved)
